fn read_into_struct(input: &str) -> (Vec<u8>, Vec<[[u8; 5]; 5]>) {
    let mut numbers: Vec<u8> = Vec::new();
    let mut boards: Vec<[[u8; 5]; 5]> = Vec::new();

    let mut i = 0;
    let mut j = 0;
    let mut new_board = [[0; 5]; 5];

    for line in input.lines() {
        if line.contains(",") {
            // first line
            line.split(",").for_each(|n| numbers.push(n.parse().expect("Error at number parsing")));
        } else if line.find(char::is_numeric) != None {
            // board
            for n in line.split_whitespace() {
                new_board[i][j] = n.parse().expect("Error at number parsing");
                j = j + 1;
            }
            i = i + 1;
            j = 0;

        } else {
            // empty Line between boards
            boards.push(new_board);
            new_board = [[0; 5]; 5];
            i = 0;
            j = 0;
        }
    }

    (numbers, boards)
}

fn board_is_bingo(numbers: &Vec<u8>, board: &[[u8; 5]; 5]) -> bool {
    // check horizontally
    for i in 0..5 {
        if numbers.contains(&board[i][0]) && numbers.contains(&board[i][1]) && numbers.contains(&board[i][2]) && numbers.contains(&board[i][3]) && numbers.contains(&board[i][4]) {
            return true;
        }
    }

    // check vertically
    for i in 0..5 {
        if numbers.contains(&board[0][i]) && numbers.contains(&board[1][i]) && numbers.contains(&board[2][i]) && numbers.contains(&board[3][i]) && numbers.contains(&board[4][i]) {
            return true;
        }
    }

    false
}

fn calculate_points(numbers: &Vec<u8>, board: &[[u8; 5]; 5]) -> usize {
    let mut sum_of_unmarked: usize = 0;
    for i in 0..5 {
        for j in 0..5 {
            if !numbers.contains(&board[i][j]) {
                sum_of_unmarked = sum_of_unmarked + board[i][j] as usize;
            }
        }
    }

    sum_of_unmarked * *numbers.last().unwrap() as usize
}

pub fn task_one(input: &str) -> String {
    let (numbers, boards) = read_into_struct(input);
    let mut drawn_numbers: Vec<u8> = Vec::new();

    for num in numbers {
        drawn_numbers.push(num);
        for board in &boards {
            if board_is_bingo(&drawn_numbers, &board) {
                return calculate_points(&drawn_numbers, &board).to_string();
            }
        }
    }

    String::new()
}

pub fn task_two(input: &str) -> String {
    let (numbers, boards) = read_into_struct(input);
    let mut drawn_numbers: Vec<u8> = Vec::new();
    let mut elimanted_boards: Vec<[[u8; 5]; 5]> = Vec::new();

    for num in numbers {
        drawn_numbers.push(num);
        for board in &boards {
            if !elimanted_boards.contains(board) {
                if board_is_bingo(&drawn_numbers, &board) {
                    if boards.len() - elimanted_boards.len() == 1 {
                        return calculate_points(&drawn_numbers, &board).to_string();
                    } else {
                        elimanted_boards.push(*board);
                    }
                }
            }
        }
    }

    String::new()
}
