const ROW_WIDTH: usize = 140;
const AMOUNT_OF_COLS: usize = 140;
// sed 's/./&\n/g' | LC_COLLATE=C sort -u | tr -d '\n' provides #$%&*+-./0123456789=@
const SYMBOLS: &[char; 10] = &['#', '$', '%', '&', '*', '+', '-', '/', '=', '@'];
const SYMBOLS_WITH_DOT: &[char; 11] = &['#', '$', '%', '&', '*', '+', '-', '/', '=', '@', '.'];

pub fn task_one(input: &str) -> String {
    let lines: Vec<&str> = input.lines().collect();
    
    let mut list_of_numbers = parse_input_into_number_list(&input);

    for number in list_of_numbers.iter_mut() {
        for coord in number.get_interesting_coords() {
            let line = lines[coord.1];
            
            if SYMBOLS.contains(&line.chars().nth(coord.0).unwrap())  {
                number.should_be_used = true;
                break;
            }
        }
    }

    list_of_numbers.iter()
        .filter(|x| x.should_be_used)
        .fold(0, |sum, x | sum + x.value )
        .to_string()
}

pub fn task_two(input: &str) -> String {
    let numbers = parse_input_into_number_list(&input);
    let lines: Vec<&str> = input.lines().collect();

    let mut sum = 0;

    for y in 0..ROW_WIDTH {
        let line = lines.get(y).unwrap();

        for x in 0..AMOUNT_OF_COLS {
            if line.chars().nth(x).unwrap() == '*' {
                let star = PartNumber { value: 1, x, y, should_be_used: false };
                let area = star.get_interesting_coords();

                let gear_numbers: Vec<usize> = numbers.iter()
                    .filter(|num| num.get_x_positions().iter().any(|x| area.contains(&(*x, num.y))))
                    .map(|num| num.value).collect();

                if gear_numbers.len() == 2 {
                    sum += gear_numbers[0] * gear_numbers[1];
                }
            }
        }
    }

    sum.to_string()
}

#[derive(Debug)]
struct PartNumber {
    value: usize,
    x: usize,
    y: usize,
    should_be_used: bool
}

impl PartNumber {
    pub fn width(&self) -> usize {
        (self.value.checked_ilog10().unwrap_or(0) + 1) as usize
    }

    pub fn get_interesting_coords(&self) -> Vec<(usize, usize)> {
        let mut coords = Vec::new();

        // row over
        if let Some(row_over) = self.y.checked_sub(1) {
            for d in 0..(self.width() + 2) {
                if let Some(x) = (self.x + d).checked_sub(1) {
                    if x < ROW_WIDTH {
                        coords.push((x, row_over));
                    }
                }
            }
        }

        // left
        if let Some(col_left) = self.x.checked_sub(1) {
            coords.push((col_left, self.y))
        }

        // right
        let col_right = self.x + self.width();
        if col_right < ROW_WIDTH {
            coords.push((col_right, self.y))
        }

        // row below
        let row_below = self.y + 1;
        if row_below < AMOUNT_OF_COLS {
            for d in 0..(self.width() + 2) {
                if let Some(x) = (self.x + d).checked_sub(1) {
                    if x < ROW_WIDTH {
                        coords.push((x, row_below));
                    }
                }
            }
        }

        coords
    }

    pub fn get_x_positions(&self) -> Vec<usize> {
        let mut positions = Vec::new();

        for d in 0..(self.width()) {
            positions.push(self.x + d);
        }

        positions
    }
}

fn parse_input_into_number_list(input: &str) -> Vec<PartNumber> {
    let mut list = Vec::new();
    let mut y = 0;

    for line in input.lines() {
        let mut local_line = line.to_string();

        for element in line.split_terminator(SYMBOLS_WITH_DOT) {
            let x: &[_] = SYMBOLS;
            let element = element.trim_matches(x);

            if let Ok(number) = element.parse::<usize>() {
                let x = local_line.find(&number.to_string()).unwrap();
                let partnumber = PartNumber { value: number, x, y, should_be_used: false };

                let mut add_string = String::new();
                for _ in  0..(partnumber.x + partnumber.width()) {
                    add_string.push('.');
                }

                local_line.replace_range(..(partnumber.x + partnumber.width()), &add_string);

                list.push(partnumber)
           }
        }
        y += 1;
    }

    list
}

#[test]
fn test_width() {
    let number_one = PartNumber {value: 9, x: 1, y: 1, should_be_used: false};
    let number_two = PartNumber {value: 20, x: 1, y: 1, should_be_used: false};
    let number_three = PartNumber {value: 800, x: 1, y: 1, should_be_used: false};
    let number_four = PartNumber {value: 8000, x: 1, y: 1, should_be_used: false};

    assert_eq!(number_one.width(), 1);
    assert_eq!(number_two.width(), 2);
    assert_eq!(number_three.width(), 3);
    assert_eq!(number_four.width(), 4);
}

#[test]
fn test_interesting_coords() {
    let number_one = PartNumber { value: 100, x: 1, y:1, should_be_used: false};
    let number_on_right = PartNumber { value: 100, x: 137, y:1, should_be_used: false};
    let number_on_left = PartNumber { value: 100, x: 0, y:1, should_be_used: false};
    let number_on_top = PartNumber { value: 100, x: 1, y:0, should_be_used: false};
    let number_on_bottom = PartNumber { value: 100, x: 1, y:139, should_be_used: false};

    assert_eq!(number_one.get_interesting_coords(),
    vec![(0,0), (1,0), (2,0), (3,0), (4,0), (0,1), (4, 1), (0, 2), (1, 2), (2, 2), (3, 2), (4, 2)]);

    assert_eq!(number_on_right.get_interesting_coords(),
    vec![(136,0), (137,0), (138,0), (139,0), (136,1), (136, 2), (137, 2), (138, 2), (139, 2)]);

    assert_eq!(number_on_left.get_interesting_coords(),
    vec![(0,0), (1,0), (2,0), (3,0), (3, 1), (0, 2), (1, 2), (2, 2), (3, 2)]);

    assert_eq!(number_on_top.get_interesting_coords(),
    vec![(0,0), (4, 0), (0, 1), (1, 1), (2, 1), (3, 1), (4, 1)]);

    assert_eq!(number_on_bottom.get_interesting_coords(),
    vec![(0,138), (1,138), (2,138), (3,138), (4,138), (0,139), (4, 139)]);
}

#[test]
fn test_task_one() {
    let input = r#"467..114..
...*......
..35..633.
......#...
617*......
.....+.58.
..592.....
......755.
...$.*....
.664.598.."#;

    assert_eq!(task_one(input), "4361");
}
