use std::collections::VecDeque;

pub fn task_one(input: &str) -> String {
    let cards = parse_input_into_card_list(&input);

    cards.iter()
        .fold(0, |sum, card| sum + card.get_score())
        .to_string()
}

pub fn task_two(input: &str) -> String {
    let cards = parse_input_into_card_list(&input);
    let mut sum = 0;

    let mut queue: VecDeque<usize> = VecDeque::new();

    for card in cards {
        let interpret_count = queue.pop_front().unwrap_or(0) + 1;
        let score = card.get_matching_numbers();

        for _ in 0..interpret_count {
            sum += 1;

            for d in 0..score {
                if let Some(aba) = queue.get(d) {
                    queue[d] = aba + 1;
                } else {
                    queue.push_back(1)
                }
            }
        }
    }

    sum.to_string()
}

#[derive(Debug)]
struct Card {
    winning_numbers: Vec<usize>,
    game_numbers: Vec<usize>,
}

impl Card {
    pub fn get_score(&self) -> usize {
        let count = self.get_matching_numbers();

        if count == 0 {
            0
        } else {
            2_usize.pow((count - 1) as u32)
        }
    }

    pub fn get_matching_numbers(&self) -> usize {
        self.game_numbers.iter()
            .filter(|num| self.winning_numbers.contains(num))
            .count()
    }
}

fn parse_input_into_card_list(input: &str) -> Vec<Card> {
    let mut cards = Vec::new();

    for line in input.lines() {
        let (_, line) = line.split_once(':').unwrap();
        let (winning_str, game_str) = line.split_once('|').unwrap();

        let winning_numbers = winning_str.split_whitespace()
            .map(|split| split.parse::<usize>().unwrap())
            .collect();

        let game_numbers = game_str.split_whitespace()
            .map(|split| split.parse::<usize>().unwrap())
            .collect();

        cards.push(Card { winning_numbers, game_numbers });
    }

    cards
}

#[test]
fn test_part_two() {
    let input = r#"Card 1: 41 48 83 86 17 | 83 86  6 31 17  9 48 53
Card 2: 13 32 20 16 61 | 61 30 68 82 17 32 24 19
Card 3:  1 21 53 59 44 | 69 82 63 72 16 21 14  1
Card 4: 41 92 73 84 69 | 59 84 76 51 58  5 54 83
Card 5: 87 83 26 28 32 | 88 30 70 12 93 22 82 36
Card 6: 31 18 13 56 72 | 74 77 10 23 35 67 36 11"#;

    let result = task_two(&input);

    assert_eq!(result, 30.to_string());
}
